package javax.swing;

import java.util.Iterator;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.Date;
import java.util.regex.Pattern;

public abstract class RowFilter<M, I>
{
    public RowFilter() {
    }
    
    private static void checkIndices(int[] array) {
        int i = array.length - 1;
        while (i >= 0) {
            if (array[i] < 0) {
                throw new IllegalArgumentException("Index must be >= 0");
            }
            --i;
        }
    }
    
    public static <M, I> RowFilter<M, I> regexFilter(String regex, int... array) {
        return new RowFilter.RegexFilter(Pattern.compile(regex), array);
    }
    
    public static <M, I> RowFilter<M, I> dateFilter(RowFilter.ComparisonType comparisonType, Date date, int... array) {
        return new RowFilter.DateFilter(comparisonType, date.getTime(), array);
    }
    
    public static <M, I> RowFilter<M, I> numberFilter(RowFilter.ComparisonType comparisonType, Number n, int... array) {
        return new RowFilter.NumberFilter(comparisonType, n, array);
    }
    
    public static <M, I> RowFilter<M, I> orFilter(Iterable<? extends RowFilter<? super M, ? super I>> iterable) {
        return new RowFilter.OrFilter(iterable);
    }
    
    public static <M, I> RowFilter<M, I> andFilter(Iterable<? extends RowFilter<? super M, ? super I>> iterable) {
        return new RowFilter.AndFilter(iterable);
    }
    
    public static <M, I> RowFilter<M, I> notFilter(RowFilter<M, I> rowFilter) {
        return new RowFilter.NotFilter(rowFilter);
    }
    
    public abstract boolean include(RowFilter.Entry<? extends M, ? extends I> p0);
    
    static void access$000(int[] array) {
        checkIndices(array);
    }
    
    public enum ComparisonType
    {
        BEFORE, 
        AFTER, 
        EQUAL, 
        NOT_EQUAL;
    }
    
    public abstract static class Entry<M, I>
    {
        public abstract M getModel();
        
        public abstract int getValueCount();
        
        public abstract Object getValue(int p0);
        
        public String getStringValue(int n) {
            Object value = this.getValue(n);
            return (value == null) ? "" : value.toString();
        }
        
        public abstract I getIdentifier();
    }
    
    private abstract static class GeneralFilter extends RowFilter<Object, Object>
    {
        private int[] columns;
        
        GeneralFilter(int[] columns) {
            RowFilter.access$000(columns);
            this.columns = columns;
        }
        
        @Override
        public boolean include(RowFilter.Entry<?, ?> entry) {
            int valueCount = entry.getValueCount();
            if (this.columns.length > 0) {
                int i = this.columns.length - 1;
                while (i >= 0) {
                    int n = this.columns[i];
                    if (n < valueCount && this.include(entry, n)) {
                        return true;
                    }
                    --i;
                }
            }
            else {
                while (--valueCount >= 0) {
                    if (!this.include(entry, valueCount)) {
                        continue;
                    }
                    return true;
                }
            }
            return false;
        }
        
        protected abstract boolean include(RowFilter.Entry<?, ?> p0, int p1);
    }
    
    private static class RegexFilter extends RowFilter.GeneralFilter
    {
        private Matcher matcher;
        
        RegexFilter(Pattern pattern, int[] array) {
            super(array);
            if (pattern == null) {
                throw new IllegalArgumentException("Pattern must be non-null");
            }
            this.matcher = pattern.matcher("");
            return;
        }
        
        @Override
        protected boolean include(RowFilter.Entry<?, ?> entry, int n) {
            this.matcher.reset(entry.getStringValue(n));
            return this.matcher.find();
        }
    }
    
    private static class DateFilter extends RowFilter.GeneralFilter
    {
        private long date;
        private RowFilter.ComparisonType type;
        
        DateFilter(RowFilter.ComparisonType type, long date, int[] array) {
            super(array);
            if (type == null) {
                throw new IllegalArgumentException("type must be non-null");
            }
            this.type = type;
            this.date = date;
            return;
        }
        
        @Override
        protected boolean include(RowFilter.Entry<?, ?> entry, int n) {
            Object value = entry.getValue(n);
            if ((value instanceof Date)) {
                long time = ((Date)value).getTime();
                switch (this.type) {
                    case BEFORE: {
                        return time < this.date;
                    }
                    case AFTER: {
                        return time > this.date;
                    }
                    case EQUAL: {
                        return time == this.date;
                    }
                    case NOT_EQUAL: {
                        return time != this.date;
                    }
                    default: {
                        break;
                    }
                }
            }
            return false;
        }
    }
    
    private static class NumberFilter extends RowFilter.GeneralFilter
    {
        private boolean isComparable;
        private Number number;
        private RowFilter.ComparisonType type;
        
        NumberFilter(RowFilter.ComparisonType type, Number number, int[] array) {
            super(array);
            if (type == null || number == null) {
                throw new IllegalArgumentException("type and number must be non-null");
            }
            this.type = type;
            this.number = number;
            this.isComparable = (number instanceof Comparable);
            return;
        }
        
        @Override
        protected boolean include(RowFilter.Entry<?, ?> entry, int n) {
            Object value = entry.getValue(n);
            if ((value instanceof Number)) {
                int n2;
                if (this.number.getClass() == value.getClass() && this.isComparable) {
                    n2 = ((Comparable)this.number).compareTo(value);
                }
                else {
                    n2 = this.longCompare((Number)value);
                }
                switch (this.type) {
                    case BEFORE: {
                        return n2 > 0;
                    }
                    case AFTER: {
                        return n2 < 0;
                    }
                    case EQUAL: {
                        return n2 == 0;
                    }
                    case NOT_EQUAL: {
                        return n2 != 0;
                    }
                    default: {
                        break;
                    }
                }
            }
            return false;
        }
        
        private int longCompare(Number n) {
            long n2 = this.number.longValue() - n.longValue();
            if (n2 < 0L) {
                return -1;
            }
            if (n2 > 0L) {
                return 1;
            }
            return 0;
        }
    }
    
    private static class OrFilter<M, I> extends RowFilter<M, I>
    {
        List<RowFilter<? super M, ? super I>> filters;
        
        OrFilter(Iterable<? extends RowFilter<? super M, ? super I>> iterable) {
            this.filters = new ArrayList();
            for (RowFilter rowFilter : iterable) {
                if (rowFilter == null) {
                    throw new IllegalArgumentException("Filter must be non-null");
                }
                this.filters.add(rowFilter);
            }
        }
        
        @Override
        public boolean include(RowFilter.Entry<? extends M, ? extends I> entry) {
            Iterator<RowFilter<? super M, ? super I>> iterator = this.filters.iterator();
            while (iterator.hasNext()) {
                if (((RowFilter)iterator.next()).include(entry)) {
                    return true;
                }
                continue;
            }
            return false;
        }
    }
    
    private static class AndFilter<M, I> extends RowFilter.OrFilter<M, I>
    {
        AndFilter(Iterable<? extends RowFilter<? super M, ? super I>> iterable) {
            super(iterable);
        }
        
        @Override
        public boolean include(RowFilter.Entry<? extends M, ? extends I> entry) {
            Iterator<RowFilter<? super M, ? super I>> iterator = this.filters.iterator();
            while (iterator.hasNext()) {
                if (!((RowFilter)iterator.next()).include(entry)) {
                    return false;
                }
                continue;
            }
            return true;
        }
    }
    
    private static class NotFilter<M, I> extends RowFilter<M, I>
    {
        private RowFilter<M, I> filter;
        
        NotFilter(RowFilter<M, I> filter) {
            if (filter == null) {
                throw new IllegalArgumentException("filter must be non-null");
            }
            this.filter = filter;
            return;
        }
        
        @Override
        public boolean include(RowFilter.Entry<? extends M, ? extends I> entry) {
            return !this.filter.include(entry);
        }
    }
}
